package main

import (
	"errors"
	"fmt"
	"github.com/fatih/color"
	"github.com/jessevdk/go-flags"
	"github.com/olekukonko/tablewriter"
	"gitlab.schukai.com/oss/libraries/go/network/traceroute"
	"os"
	"strconv"
	"time"
)

const (
	exitOK    = 0  /* successful termination */
	exitUsage = 64 /* command line usage error */
	exitIOErr = 74 /* input/output error */
)

var opts struct {
	Realtime  bool   `long:"realtime" short:"r" description:"show the steps in real time"`
	Format    string `long:"format" short:"f" description:"output format, is not valid for real time mode." choice:"table" default:"table"`
	Hops      int    `long:"hops" short:"h" description:"maximal hops."`
	Threshold int    `long:"threshold" short:"m" description:"from the threshold value in milliseconds the value is marked in red" default:"20"`
}

var parser *flags.Parser

func round(d time.Duration, digits int) time.Duration {

	var divs = []time.Duration{
		time.Duration(1), time.Duration(10), time.Duration(100), time.Duration(1000)}

	switch {
	case d > time.Second:
		d = d.Round(time.Second / divs[digits])
	case d > time.Millisecond:
		d = d.Round(time.Millisecond / divs[digits])
	case d > time.Microsecond:
		d = d.Round(time.Microsecond / divs[digits])
	}
	return d
}

func printAsTable(result traceroute.Results) {

	table := tablewriter.NewWriter(os.Stdout)
	table.SetHeader([]string{"#", "host", "time"})
	table.SetColumnAlignment([]int{tablewriter.ALIGN_RIGHT, tablewriter.ALIGN_LEFT, tablewriter.ALIGN_RIGHT})

	table.NumLines()
	table.SetBorders(tablewriter.Border{
		Left:   false,
		Right:  false,
		Top:    false,
		Bottom: false,
	})

	d := time.Duration(opts.Threshold)
	marker := d * time.Millisecond

	for _, i := range result.Hops {

		r := []string{
			strconv.FormatInt(int64(i.Hop), 10),
			i.Station,
			fmt.Sprintf("%s", round(i.Latency, 3))}

		if i.Latency > marker {
			table.Rich(r, []tablewriter.Colors{tablewriter.Colors{}, tablewriter.Colors{}, tablewriter.Colors{tablewriter.Bold, tablewriter.FgRedColor}})
		} else {
			table.Append(r)
		}

	}

	table.Render() // Send output

}

func printErrorAndExit(err error, exitcode int) {

	out := color.New(color.FgRed).Add(color.Underline)

	fmt.Println()
	out.Println(err.Error())

	fmt.Println()
	parser.WriteHelp(os.Stdout)
	fmt.Println()
	os.Exit(exitcode)
}

func main() {

	parser = flags.NewParser(&opts, flags.HelpFlag|flags.PrintErrors)
	parser.Usage = "is the way to measure the path to a host."
	parser.ShortDescription = "Traceroute to host."
	parser.LongDescription = "With this command the path from the localhost to the\n" +
		"target computer can be recorded and the duration of the\n" +
		"packet runtimes can be measured. Depending on the IP\n" +
		"of the destination, either a V4 or V6 measurement is performed."

	args, err := parser.Parse()

	if len(args) != 1 || args[0] == "" {
		printErrorAndExit(errors.New("the required `host' was not specified"), exitUsage)
	}

	session, err := traceroute.NewSession(args[0])
	if err != nil {
		printErrorAndExit(err, exitIOErr)
		return
	}

	if opts.Hops > 0 {
		session.MaxHops = opts.Hops
	}

	if opts.Realtime {
		session.CallBack = func(result traceroute.Result) {
			if result.Err != nil {
				fmt.Printf("%v\t%s\t\t\t\t%v\t\t(%s)\n", result.Hop, result.Station, result.Latency, result.Err.Error())
				return
			}

			fmt.Printf("%v\t%s\t\t\t%v\n", result.Hop, result.Station, result.Latency)
		}
	}

	results, err := session.TraceRoute()

	if err != nil {
		printErrorAndExit(err, exitIOErr)
	}

	if opts.Realtime {
		os.Exit(exitOK)
	}
	
	switch opts.Format {
	case "table":
		printAsTable(*results)
	default:
		printErrorAndExit(errors.New("unsupported format"), exitUsage)
	}

	os.Exit(exitOK)
}
